from bscscan.accounts import Account
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

address = ['0xddbd2b932c763ba5b1b7ae3b362eac3e8d40121a',
           '0xddbd2b932c763ba5b1b7ae3b362eac3e8d40121a']

api = Account(address=address, api_key=key)
balances = api.get_balance_multiple()
print(balances)



from bscscan.tokens import Tokens
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

address = '0x043d9b3af52b623e54a3dF92F1682757eb29F912'
api = Tokens(contract_address='0x0cf2b5aabf844b49480dbf5e6192b873a865fae4',
             api_key=key)
balance = api.get_token_balance(address=address)
print(balance)



from bscscan.accounts import Account
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

address = '0xddbd2b932c763ba5b1b7ae3b362eac3e8d40121a'

api = Account(address=address, api_key=key)
transactions = api.get_transaction_page(page=1, offset=10000, sort='des', erc20=True)
print(transactions)



from .client import Client
import re


class Account(Client):
    PAGE_NUM_PATTERN = re.compile(
        r'[1-9](?:\d{0,2})(?:,\d{3})*(?:\.\d*[1-9])?|0?\.\d*[1-9]|0')

    def __init__(self, address=Client.dao_address, api_key='YourApiKeyToken'):
        Client.__init__(self, address=address, api_key=api_key)
        self.url_dict[self.MODULE] = 'account'

    def get_balance(self):
        self.url_dict[self.ACTION] = 'balance'
        self.url_dict[self.TAG] = 'latest'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_balance_multiple(self):
        self.url_dict[self.ACTION] = 'balancemulti'
        self.url_dict[self.TAG] = 'latest'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_transaction_page(self, page=1, offset=10000, sort='asc',
                             internal=False, erc20=False) -> list:
        """
        Get a page of transactions, each transaction
        returns list of dict with keys:
            nonce
            hash
            cumulativeGasUsed
            gasUsed
            timeStamp
            blockHash
            value (in wei)
            input
            gas
            isInternalTx
            contractAddress
            confirmations
            gasPrice
            transactionIncex
            to
            from
            isError
            blockNumber

        sort options:
            'asc' -> ascending order
            'desc' -> descending order

        internal options: (currently marked at Beta for bscscan.io)
            True  -> Gets the internal transactions of the address
            False -> (default) get normal external transactions

        erc20 options: (currently marked at Beta for bscscan.io)
            True  -> Gets the erc20 token transcations of the address
            False -> (default) get normal external transactions

        NOTE: not sure if this works for contract addresses, requires testing
        """
        if internal:
            self.url_dict[self.ACTION] = 'txlistinternal'
        elif erc20:
            self.url_dict[self.ACTION] = 'tokentx'
        else:
            self.url_dict[self.ACTION] = 'txlist'
        self.url_dict[self.PAGE] = str(page)
        self.url_dict[self.OFFSET] = str(offset)
        self.url_dict[self.SORT] = sort
        self.build_url()
        req = self.connect()
        return req['result']

    def get_all_transactions(self, offset=10000, sort='asc',
                             internal=False) -> list:
        if internal:
            self.url_dict[self.ACTION] = 'txlistinternal'
        else:
            self.url_dict[self.ACTION] = 'txlist'
        self.url_dict[self.PAGE] = str(1)
        self.url_dict[self.OFFSET] = str(offset)
        self.url_dict[self.SORT] = sort
        self.build_url()

        trans_list = []
        while True:
            self.build_url()
            req = self.connect()
            if "No transactions found" in req['message']:
                print(
                    "Total number of transactions: {}".format(len(trans_list)))
                self.page = ''
                return trans_list
            else:
                trans_list += req['result']
                # Find any character block that is a integer of any length
                page_number = re.findall(Account.PAGE_NUM_PATTERN,
                                         self.url_dict[self.PAGE])
                print("page {} added".format(page_number[0]))
                self.url_dict[self.PAGE] = str(int(page_number[0]) + 1)

    def get_blocks_mined_page(self, blocktype='blocks', page=1,
                              offset=10000) -> list:
        """
        Get a page of blocks mined by given address,
         returns list of dict with keys:
            blockReward (in wei)
            blockNumber
            timeStamp

        blocktype options:
            'blocks' -> full blocks only
            'uncles' -> uncles only
        """
        self.url_dict[self.ACTION] = 'getminedblocks'
        self.url_dict[self.BLOCK_TYPE] = blocktype
        self.url_dict[self.PAGE] = str(page)
        self.url_dict[self.OFFSET] = str(offset)
        self.build_url()
        req = self.connect()
        return req['result']

    def get_all_blocks_mined(self, blocktype='blocks', offset=10000) -> list:
        self.url_dict[self.ACTION] = 'getminedblocks'
        self.url_dict[self.BLOCK_TYPE] = blocktype
        self.url_dict[self.PAGE] = str(1)
        self.url_dict[self.OFFSET] = str(offset)
        blocks_list = []
        while True:
            self.build_url()
            req = self.connect()
            print(req['message'])
            if "No transactions found" in req['message']:
                print(
                    "Total number of blocks mined: {}".format(
                        len(blocks_list)))
                return blocks_list
            else:
                blocks_list += req['result']
                # Find any character block that is a integer of any length
                page_number = re.findall(Account.PAGE_NUM_PATTERN,
                                         self.url_dict[self.PAGE])
                print("page {} added".format(page_number[0]))
                self.url_dict[self.PAGE] = str(int(page_number[0]) + 1)

    def get_internal_by_hash(self, tx_hash=''):
        """
        Currently not implemented
        :return:
        """
        pass

    def update_transactions(self, address, trans):
        """
        Gets last page of transactions (last 10k trans)
        and updates current trans book (book)
        """
        pass



from bscscan.accounts import Account
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

address = '0x043d9b3af52b623e54a3dF92F1682757eb29F912'

api = Account(address=address, api_key=key)
transactions = api.get_all_transactions(offset=100, sort='asc', internal=False)

print(transactions[0])



from .client import Client


class Contract(Client):
    def __init__(self, address=Client.dao_address, api_key='YourApiKeyToken'):
        Client.__init__(self, address=address, api_key=api_key)
        self.url_dict[self.MODULE] = 'contract'

    def get_abi(self):
        self.url_dict[self.ACTION] = 'getabi'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_sourcecode(self):
        self.url_dict[self.ACTION] = 'getsourcecode'
        self.build_url()
        req = self.connect()
        return req['result']



from .client import Client


class Stats(Client):
    def __init__(self, api_key='YourApiKeyToken'):
        Client.__init__(self, address='', api_key=api_key)
        self.url_dict[self.MODULE] = 'stats'

    def get_total_bnb_supply(self):
        self.url_dict[self.ACTION] = 'bnbsupply'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_validators(self):
        self.url_dict[self.ACTION] = 'validators'
        self.build_url()
        req = self.connect()
        return req['result']

    # NOT YET AVAILABLE IN BSCSCAN APIS
    # def get_bnb_last_price(self):
    #     self.url_dict[self.ACTION] = 'bnbprice'
    #     self.build_url()
    #     req = self.connect()
    #     return req['result']



from bscscan.stats import Stats
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

# call with default address, The DAO
api = Stats(api_key=key)
supply = api.get_total_bnb_supply()
print(supply)



from bscscan.tokens import Tokens
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

#  tokenname options are:
#     DGD
#     MKR
#     TheDAO
api = Tokens(tokenname='SNT', api_key=key)
supply = api.get_total_supply()
print(supply)



from bscscan.accounts import Account
import json

with open('../../api_key.json', mode='r') as key_file:
    key = json.loads(key_file.read())['key']

address = '0x2a65aca4d5fc5b5c859090a6c34d164135398226'

api = Account(address=address, api_key=key)
blocks = api.get_all_blocks_mined(offset=10000, blocktype='uncles')
print(blocks)



# from bscscan.stats import Stats
# import json
#
# with open('../../api_key.json', mode='r') as key_file:
#     key = json.loads(key_file.read())['key']
#
# api = Stats(api_key=key)
# last_price = api.get_bnb_last_price()
# print(last_price)



from .client import Client


class Tokens(Client):
    def __init__(self, contract_address, api_key='YourApiKeyToken'):
        Client.__init__(self, address='', api_key=api_key)
        # self.url_dict[self.TOKEN_NAME] = tokenname
        self.url_dict[self.CONTRACT_ADDRESS] = contract_address

    def get_total_supply(self):
        self.url_dict[self.ACTION] = 'tokensupply'
        self.url_dict[self.MODULE] = 'stats'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_circulating_supply(self):
        self.url_dict[self.ACTION] = 'tokenCsupply'
        self.url_dict[self.MODULE] = 'stats'
        self.build_url()
        req = self.connect()
        return req['result']

    def get_token_balance(self, address):
        self.url_dict[self.ADDRESS] = address
        self.url_dict[self.MODULE] = 'account'
        self.url_dict[self.ACTION] = 'tokenbalance'
        self.build_url()
        req = self.connect()
        return req['result']



import setuptools

setuptools.setup(
    name='py_bscscan_api',
    version='0.0.1',
    packages=['examples', 'examples.stats', 'examples.tokens',
              'examples.accounts', 'bscscan'],
    url='https://github.com/darkzeb/py-bscscan-api',
    license='MIT',
    author='darkzeb',
    author_email='bbezkradd@gmail.com',
    description='Python Bindings to Bscscan.com API',
    install_requires=[
        'requests>=2.20.0',
    ],
    classifiers=[
        "Programming Language :: Python :: 3"
    ]
)



# coding: utf-8
import collections

import requests


class ClientException(Exception):
    """Unhandled API client exception"""
    message = 'unhandled error'

    def __init__(self, message=None):
        if message is not None:
            self.message = message

    def __unicode__(self):
        return u'<Err: {0.message}>'.format(self)

    __str__ = __unicode__


class ConnectionRefused(ClientException):
    """Connection refused by remote host"""


class EmptyResponse(ClientException):
    """Empty response from API"""


class BadRequest(ClientException):
    """Invalid request passed"""


#  Assume user puts his API key in the api_key.json
#  file under variable name "key"
class Client(object):
    dao_address = '0xbb9bc244d798123fde783fcc1c72d3bb8c189413'

    # Constants
    PREFIX = 'https://api.bscscan.com/api?'
    MODULE = 'module='
    ACTION = '&action='
    CONTRACT_ADDRESS = '&contractaddress='
    ADDRESS = '&address='
    OFFSET = '&offset='
    PAGE = '&page='
    SORT = '&sort='
    BLOCK_TYPE = '&blocktype='
    TO = '&to='
    VALUE = '&value='
    DATA = '&data='
    POSITION = '&position='
    HEX = '&hex='
    GAS_PRICE = '&gasPrice='
    GAS = '&gas='
    START_BLOCK = '&startblock='
    END_BLOCK = '&endblock='
    BLOCKNO = '&blockno='
    TXHASH = '&txhash='
    TAG = '&tag='
    BOOLEAN = '&boolean='
    INDEX = '&index='
    API_KEY = '&apikey='

    url_dict = {}

    def __init__(self, address, api_key=''):
        self.http = requests.session()
        self.url_dict = collections.OrderedDict([
            (self.MODULE, ''),
            (self.ADDRESS, ''),
            (self.OFFSET, ''),
            (self.PAGE, ''),
            (self.SORT, ''),
            (self.BLOCK_TYPE, ''),
            (self.TO, ''),
            (self.VALUE, ''),
            (self.DATA, ''),
            (self.POSITION, ''),
            (self.HEX, ''),
            (self.GAS_PRICE, ''),
            (self.GAS, ''),
            (self.START_BLOCK, ''),
            (self.END_BLOCK, ''),
            (self.BLOCKNO, ''),
            (self.TXHASH, ''),
            (self.TAG, ''),
            (self.BOOLEAN, ''),
            (self.INDEX, ''),
            (self.API_KEY, api_key)])

        # Var initialization should take place within init
        self.url = None

        self.check_and_get_api()

        if (len(address) > 20) and (type(address) == list):
            raise BadRequest("Bscscan only takes 20 addresses at a time")
        elif (type(address) == list) and (len(address) <= 20):
            self.url_dict[self.ADDRESS] = ','.join(address)
        else:
            self.url_dict[self.ADDRESS] = address

    def build_url(self):
        self.url = self.PREFIX + ''.join(
            [param + val if val else '' for param, val in
             self.url_dict.items()])

    def connect(self):
        # TODO: deal with "unknown exception" error
        try:
            req = self.http.get(self.url)
        except requests.exceptions.ConnectionError:
            raise ConnectionRefused

        if req.status_code == 200:
            # Check for empty response
            if req.text:
                data = req.json()
                status = data.get('status')
                if status == '1' or self.check_keys_api(data):
                    return data
                else:
                    raise EmptyResponse(data.get('message', 'no message'))
        raise BadRequest(
            "Problem with connection, status code: %s" % req.status_code)

    def check_and_get_api(self):
        if self.url_dict[self.API_KEY]:  # Check if api_key is empty string
            pass
        else:
            self.url_dict[self.API_KEY] = input(
                'Please type your EtherScan.io API key: ')

    @staticmethod
    def check_keys_api(data):
        return all(k in data for k in ('jsonrpc', 'id', 'result'))
